@@ -5,11 +5,7 @@ from itertools import chain
 from types import FunctionType
 from typing import Tuple, List, Union, Optional
 
-# noinspection PyProtectedMember
-from azure.kusto.data._response import KustoResponseDataSet
-from azure.kusto.data.helpers import dataframe_from_result_table
-
-from pykusto.client import Table
+from pykusto.client import Table, KustoResponse
 from pykusto.expressions import BooleanType, ExpressionType, AggregationExpression, OrderedType, \
     StringType, AssignmentBase, AssignmentFromAggregationToColumn, AssignmentToSingleColumn, AnyTypeColumn, \
     BaseExpression, \
@@ -109,7 +105,9 @@ class Query:
     def join(self, query: 'Query', kind: JoinKind = None) -> 'JoinQuery':
         return JoinQuery(self, query, kind)
 
-    def project(self, *args: Union[AnyTypeColumn, AssignmentBase, BaseExpression], **kwargs: ExpressionType) -> 'ProjectQuery':
+    def project(
+            self, *args: Union[AnyTypeColumn, AssignmentBase, BaseExpression], **kwargs: ExpressionType
+    ) -> 'ProjectQuery':
         columns: List[AnyTypeColumn] = []
         assignments: List[AssignmentBase] = []
         for arg in args:
@@ -117,10 +115,9 @@ class Query:
                 columns.append(arg)
             elif isinstance(arg, AssignmentBase):
                 assignments.append(arg)
-            elif isinstance(arg, BaseExpression):
-                assignments.append(arg.assign_to())
             else:
-                raise ValueError("Invalid assignment: " + arg.to_kql())
+                assert isinstance(arg, BaseExpression), "Invalid assignment"
+                assignments.append(arg.assign_to())
         for column_name, expression in kwargs.items():
             assignments.append(AssignmentToSingleColumn(AnyTypeColumn(column_name), expression))
         return ProjectQuery(self, columns, assignments)
@@ -160,16 +157,17 @@ class Query:
         for arg in args:
             if isinstance(arg, AggregationExpression):
                 assignments.append(arg.assign_to())
-            elif isinstance(arg, AssignmentFromAggregationToColumn):
-                assignments.append(arg)
             else:
-                raise ValueError("Invalid assignment: " + str(arg))
+                assert isinstance(arg, AssignmentFromAggregationToColumn), "Invalid assignment"
+                assignments.append(arg)
         for column_name, agg in kwargs.items():
             assignments.append(AssignmentFromAggregationToColumn(AnyTypeColumn(column_name), agg))
         return SummarizeQuery(self, assignments)
 
-    def mv_expand(self, *columns: Union[AnyTypeColumn, ColumnToType], bag_expansion: BagExpansion = None, with_item_index: AnyTypeColumn = None,
-                  limit: int = None) -> 'MvExpandQuery':
+    def mv_expand(
+            self, *columns: Union[AnyTypeColumn, ColumnToType], bag_expansion: BagExpansion = None,
+            with_item_index: AnyTypeColumn = None, limit: int = None
+    ) -> 'MvExpandQuery':
         if len(columns) == 0:
             raise ValueError("Please specify one or more columns for mv-expand")
         return MvExpandQuery(self, columns, bag_expansion, with_item_index, limit)
@@ -180,11 +178,16 @@ class Query:
     def evaluate(self, plugin_name, *args: ExpressionType, distribution: Distribution = None) -> 'EvaluateQuery':
         return EvaluateQuery(self, plugin_name, *args, distribution=distribution)
 
-    def evaluate_udf(self, udf: FunctionType, extend: bool = True, distribution: Distribution = None, **type_specs: KustoType) -> 'EvaluateQuery':
+    def evaluate_udf(
+            self, udf: FunctionType, extend: bool = True, distribution: Distribution = None, **type_specs: KustoType
+    ) -> 'EvaluateQuery':
         return EvaluateQuery(
             self, 'python',
-            BaseExpression(
-                KQL('typeof({})'.format(('*, ' if extend else '') + ', '.join(field_name + ':' + kusto_type.primary_name for field_name, kusto_type in type_specs.items())))),
+            AnyExpression(
+                KQL('typeof({})'.format(('*, ' if extend else '') + ', '.join(
+                    field_name + ':' + kusto_type.primary_name for field_name, kusto_type in type_specs.items()
+                )))
+            ),
             stringify_python_func(udf),
             distribution=distribution
         )
@@ -196,7 +199,7 @@ class Query:
 
     @abstractmethod
     def _compile(self) -> KQL:
-        pass
+        raise NotImplementedError()  # pragma: no cover
 
     def _compile_all(self, use_full_table_name) -> KQL:
         if self._head is None:
@@ -205,9 +208,9 @@ class Query:
             else:
                 table = self._table
                 if use_full_table_name:
-                    return table.get_full_table()
+                    return table.to_query_format(fully_qualified=True)
                 else:
-                    return table.get_table()
+                    return table.to_query_format()
         else:
             return KQL("{} | {}".format(self._head._compile_all(use_full_table_name), self._compile()))
 
@@ -222,11 +225,11 @@ class Query:
         logger.debug("Complied query: " + result)
         return result
 
-    def execute(self, table: Table = None) -> KustoResponseDataSet:
+    def execute(self, table: Table = None) -> KustoResponse:
         if self.get_table() is None:
             if table is None:
                 raise RuntimeError("No table supplied")
-            rendered_query = table.get_table() + self.render()
+            rendered_query = table.to_query_format() + self.render()
         else:
             if table is not None:
                 raise RuntimeError("This table is already bound to a query")
@@ -237,8 +240,7 @@ class Query:
         return table.execute(rendered_query)
 
     def to_dataframe(self, table: Table = None):
-        res = self.execute(table)
-        return dataframe_from_result_table(res.primary_results[0])
+        return self.execute(table).to_dataframe()
 
 
 class ProjectQuery(Query):
@@ -276,10 +278,7 @@ class ProjectAwayQuery(Query):
         self._columns = columns
 
     def _compile(self) -> KQL:
-        def col_or_wildcard_to_string(col_or_wildcard):
-            return col_or_wildcard.kql if isinstance(col_or_wildcard, AnyTypeColumn) else col_or_wildcard
-
-        return KQL('project-away {}'.format(', '.join((col_or_wildcard_to_string(c) for c in self._columns))))
+        return KQL('project-away {}'.format(', '.join((str(c) for c in self._columns))))
 
 
 class DistinctQuery(Query):
@@ -384,7 +383,7 @@ class _OrderQueryBase(Query):
         return self
 
     @staticmethod
-    def _compile_order_spec(order_spec):
+    def _compile_order_spec(order_spec: OrderSpec) -> str:
         res = str(order_spec.col.kql)
         if order_spec.order is not None:
             res += " " + str(order_spec.order.value)
@@ -479,10 +478,9 @@ class SummarizeQuery(Query):
         for arg in args:
             if isinstance(arg, AnyTypeColumn) or isinstance(arg, BaseExpression):
                 self._by_columns.append(arg)
-            elif isinstance(arg, AssignmentToSingleColumn):
-                self._by_assignments.append(arg)
             else:
-                raise ValueError("Invalid assignment: " + arg.to_kql())
+                assert isinstance(arg, AssignmentToSingleColumn), "Invalid assignment"
+                self._by_assignments.append(arg)
         for column_name, group_exp in kwargs.items():
             self._by_assignments.append(AssignmentToSingleColumn(AnyTypeColumn(column_name), group_exp))
         return self
@@ -503,7 +501,10 @@ class MvExpandQuery(Query):
     _with_item_index: AnyTypeColumn
     _limit: int
 
-    def __init__(self, head: Query, columns: Tuple[Union[AnyTypeColumn, ColumnToType]], bag_expansion: BagExpansion, with_item_index: AnyTypeColumn, limit: int):
+    def __init__(
+            self, head: Query, columns: Tuple[Union[AnyTypeColumn, ColumnToType]], bag_expansion: BagExpansion,
+            with_item_index: AnyTypeColumn, limit: int
+    ):
         super(MvExpandQuery, self).__init__(head)
         self._columns = columns
         self._bag_expansion = bag_expansion
